#!/usr/bin/perl -w
# (c) Copyright 2005-2006. CodeWeavers, Inc.
package CXMenuKDE;
use strict;
use CXLog;
use CXMenuKG;
use base "CXMenuKG";

my $kde_extension;
my $kde_section_name;
sub new($$$)
{
    my ($class, $tag, $gui_info)=@_;

    if (!defined $kde_extension)
    {
        if ($gui_info->{kde_version} =~ /^1\./)
        {
            $kde_extension="kdelnk";
            $kde_section_name="KDE Desktop Entry";
        }
        else
        {
            $kde_extension="desktop";
            $kde_section_name="Desktop Entry";
        }
    }

    my $self={
        tag             => $tag,
        encoding        => "UTF-8",
        escaping        => "dollar",
        extension       => $kde_extension,
        extensions      => "\\.(desktop|kdelnk)",
        section_name    => $kde_section_name
    };
    bless $self, $class;
    return $self;
}

sub detect($$$$)
{
    my ($class, $cxoptions, $cxconfig, $gui_info)=@_;
    return () if (!$gui_info->{kde_on});

    my @selves;
    # Create a separate instance for the menu and desktop
    # so they can be disabled independently.
    if ($gui_info->{kde_preferred_menu})
    {
        my $self=new($class, $cxoptions->{tag}, $gui_info);
        $self->{menu}         = $gui_info->{kde_preferred_menu};
        push @selves, $self;
    }
    if ($gui_info->{kde_preferred_alt_menu})
    {
        my $self=new($class, $cxoptions->{tag}, $gui_info);
        $self->{menu}         = $gui_info->{kde_preferred_alt_menu};
        push @selves, $self;
    }
    if ($gui_info->{kde_preferred_desktop})
    {
        my $self=new($class, $cxoptions->{tag}, $gui_info);
        $self->{desktop}      = $gui_info->{kde_preferred_desktop};
        push @selves, $self;
    }
    my $legacy_dir="/etc/opt/kde3/share/applnk";
    if (-d $legacy_dir and -w $legacy_dir)
    {
        # CrossOver <= 2.1 used this and now we must cleanup after it.
        # We only have to deal with this in --removeall --pattern "legacy".
        my $self=new($class, $cxoptions->{tag}, $gui_info);
        $self->{menu}         = $legacy_dir;
        $self->{remove_only}  = 1;
        $self->{legacy_only}  = 1;
        push @selves, $self;
    }
    if (-w $gui_info->{kde_global_desktoplinks})
    {
        # CrossOver <= 5.0.1 used this and now we must cleanup after it.
        # We only have to deal with this when removing desktop icons.
        my $self=new($class, $cxoptions->{tag}, $gui_info);
        $self->{desktop}      = $gui_info->{kde_global_desktoplinks};
        $self->{remove_only}  = 1;
        push @selves, $self;
    }

    return @selves;
}

return 1;
