#!/usr/bin/perl -w
# (c) Copyright 2005-2008. CodeWeavers, Inc.
package CXMenuWindows;
use strict;

use CXLog;
use CXUtils;
use CXMenu;

sub detect($$$$)
{
    my ($class, $cxoptions)=@_;

    my $self={
        tag         => $cxoptions->{tag},
        desktopdata => $cxoptions->{desktopdata}
    };
    bless $self, $class;
    return ($self);
}

sub id($)
{
    return "CXMenuWindows/";
}

my $wine_script;
sub get_command($;$)
{
    my ($menu, $background)=@_;

    my $lnk=CXMenu::menu_to_lnk($menu->{rawpath});

    # 'wine --start' really expects the parameter to be in the default encoding
    require CXRecode;
    $lnk=CXRecode::to_sys("UTF-8", $lnk);

    if (!defined $wine_script)
    {
        $wine_script="$ENV{CX_ROOT}/bin/wine-remote";
        if (-x $wine_script)
        {
            $wine_script=shquote_string($wine_script) . " --file-is-last";
        }
        else
        {
            $wine_script=shquote_string("$ENV{CX_ROOT}/bin/wine");
        }
    }
    return join(" ", $wine_script,
                "--bottle", shquote_string($ENV{CX_BOTTLE}),
                "--check", ($background ? "" : "--wait-children"),
                "--start", shquote_string($lnk));
}

sub install($$)
{
    my ($self, $components)=@_;

    my $menu=@$components[-1];
    return 1 if ($menu->{type} ne "windows" or $menu->{is_dir});

    if (!defined $ENV{WINEPREFIX})
    {
        cxerr("cannot create the '$menu->{rawpath}' Windows menu because no bottle was specified\n");
        return -1; # Fatal error
    }
    if ($menu->{command} ne "")
    {
        cxwarn("'windows' menus cannot have a command. Overriding the command\n");
    }

    # Remove the parent directory from the garbage collection list
    my $path=mangle_path($menu->{rawpath});
    delete $self->{garbage_collect}->{cxdirname($path)};

    my $script="$self->{desktopdata}/cxmenu/$path";
    cxlog("Creating '$script'\n");

    my $scriptdir=cxdirname($script);
    if (!-d $scriptdir and !cxmkpath($scriptdir))
    {
        cxerr("unable to create '$scriptdir': $@\n");
        return -1; # Fatal error
    }

    chmod 0700, $script;
    my $fh;
    if (!open($fh, ">", $script))
    {
        cxerr("unable to open '$script' for writing: $!\n");
        return -1; # Fatal error
    }
    print $fh "#!/bin/sh\n";
    print $fh "exec ", get_command($menu),  " \"\$\@\"\n";
    close($fh);
    my $mode=0755 & ~umask();
    chmod $mode, $script;

    $menu->{command}=shquote_string($script);
    $menu->{icon}=CXMenu::get_icon_path($ENV{WINEPREFIX}, $menu->{rawpath});
    $menu->{genericname}="Windows Application (CrossOver)";
    return 1;
}

sub query($$)
{
    # Don't report the CXMenuWindows install status
    return ("", "");
}

sub uninstall($$)
{
    my ($self, $components)=@_;

    my $menu=@$components[-1];
    return 1 if ($menu->{is_dir});
    return 1 if ($menu->{type} ne "windows" and !defined $ENV{WINEPREFIX});
    # Do the uninstall for non-Windows menus so we clean up after them
    # in case their type changed

    if (!defined $ENV{WINEPREFIX})
    {
        cxerr("cannot remove the script associated to the '$menu->{rawpath}' Windows menu because no bottle was specified\n");
        return 0;
    }

    my $path=mangle_path($menu->{rawpath});
    my $script="$self->{desktopdata}/cxmenu/$path";
    if (-f $script)
    {
        cxlog("Deleting '$script'\n");
        if (!unlink $script)
        {
            cxwarn("unable to delete '$script': $!\n");
        }
    }

    # Also mark the parent directory for deletion
    $self->{garbage_collect}->{cxdirname($path)}=1;

    return 1;
}

sub removeall($$)
{
    my ($self, $pattern)=@_;

    if (defined $ENV{WINEPREFIX} and $pattern ne "legacy" and
        ($self->{tag} || "") =~ /^$pattern/)
    {
        foreach my $dir ("$self->{desktopdata}/cxmenu/Desktop",
                         "$self->{desktopdata}/cxmenu/StartMenu")
        {
            if (-d $dir)
            {
                cxlog("Deleting the '$dir' directory\n");
                require File::Path;
                if (!File::Path::rmtree($dir))
                {
                    cxerr("unable to delete the '$dir' directory: $!\n");
                }
            }
        }
        CXUtils::garbage_collect_subdirs($self->{desktopdata}, "/cxmenu", 1);
    }
    return 1;
}

sub finalize($)
{
    my ($self)=@_;
    return 1 if (!defined $ENV{WINEPREFIX});

    my $root="$self->{desktopdata}/cxmenu";
    foreach my $path (sort { $b cmp $a } keys %{$self->{garbage_collect}})
    {
        CXUtils::garbage_collect_subdirs($root, $path, 1);
    }
    CXUtils::garbage_collect_subdirs($self->{desktopdata}, "/cxmenu", 1);
    return 1;
}

return 1;
